from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="ImportOpenData")


@_attrs_define
class ImportOpenData:
    """
    Attributes:
        selected_version (str):
        selected_datatypes (list[str]):
        update_existing (bool | Unset):  Default: True.
        use_metric (bool | Unset):  Default: True.
    """

    selected_version: str
    selected_datatypes: list[str]
    update_existing: bool | Unset = True
    use_metric: bool | Unset = True
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        selected_version = self.selected_version

        selected_datatypes = self.selected_datatypes

        update_existing = self.update_existing

        use_metric = self.use_metric

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "selected_version": selected_version,
                "selected_datatypes": selected_datatypes,
            }
        )
        if update_existing is not UNSET:
            field_dict["update_existing"] = update_existing
        if use_metric is not UNSET:
            field_dict["use_metric"] = use_metric

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("selected_version", (None, str(self.selected_version).encode(), "text/plain")))

        for selected_datatypes_item_element in self.selected_datatypes:
            files.append(("selected_datatypes", (None, str(selected_datatypes_item_element).encode(), "text/plain")))

        if not isinstance(self.update_existing, Unset):
            files.append(("update_existing", (None, str(self.update_existing).encode(), "text/plain")))

        if not isinstance(self.use_metric, Unset):
            files.append(("use_metric", (None, str(self.use_metric).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        selected_version = d.pop("selected_version")

        selected_datatypes = cast(list[str], d.pop("selected_datatypes"))

        update_existing = d.pop("update_existing", UNSET)

        use_metric = d.pop("use_metric", UNSET)

        import_open_data = cls(
            selected_version=selected_version,
            selected_datatypes=selected_datatypes,
            update_existing=update_existing,
            use_metric=use_metric,
        )

        import_open_data.additional_properties = d
        return import_open_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.import_open_data_version_meta_data import ImportOpenDataVersionMetaData


T = TypeVar("T", bound="ImportOpenDataMetaData")


@_attrs_define
class ImportOpenDataMetaData:
    """
    Attributes:
        versions (list[str]):
        datatypes (list[str]):
        base (ImportOpenDataVersionMetaData):
        cs (ImportOpenDataVersionMetaData):
        da (ImportOpenDataVersionMetaData):
        de (ImportOpenDataVersionMetaData):
        el (ImportOpenDataVersionMetaData):
        en (ImportOpenDataVersionMetaData):
        es (ImportOpenDataVersionMetaData):
        fr (ImportOpenDataVersionMetaData):
        hu (ImportOpenDataVersionMetaData):
        it (ImportOpenDataVersionMetaData):
        nb_no (ImportOpenDataVersionMetaData):
        nl (ImportOpenDataVersionMetaData):
        pl (ImportOpenDataVersionMetaData):
        pt (ImportOpenDataVersionMetaData):
        pt_br (ImportOpenDataVersionMetaData):
        sk (ImportOpenDataVersionMetaData):
        sl (ImportOpenDataVersionMetaData):
        zh_hans (ImportOpenDataVersionMetaData):
    """

    versions: list[str]
    datatypes: list[str]
    base: ImportOpenDataVersionMetaData
    cs: ImportOpenDataVersionMetaData
    da: ImportOpenDataVersionMetaData
    de: ImportOpenDataVersionMetaData
    el: ImportOpenDataVersionMetaData
    en: ImportOpenDataVersionMetaData
    es: ImportOpenDataVersionMetaData
    fr: ImportOpenDataVersionMetaData
    hu: ImportOpenDataVersionMetaData
    it: ImportOpenDataVersionMetaData
    nb_no: ImportOpenDataVersionMetaData
    nl: ImportOpenDataVersionMetaData
    pl: ImportOpenDataVersionMetaData
    pt: ImportOpenDataVersionMetaData
    pt_br: ImportOpenDataVersionMetaData
    sk: ImportOpenDataVersionMetaData
    sl: ImportOpenDataVersionMetaData
    zh_hans: ImportOpenDataVersionMetaData
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        versions = self.versions

        datatypes = self.datatypes

        base = self.base.to_dict()

        cs = self.cs.to_dict()

        da = self.da.to_dict()

        de = self.de.to_dict()

        el = self.el.to_dict()

        en = self.en.to_dict()

        es = self.es.to_dict()

        fr = self.fr.to_dict()

        hu = self.hu.to_dict()

        it = self.it.to_dict()

        nb_no = self.nb_no.to_dict()

        nl = self.nl.to_dict()

        pl = self.pl.to_dict()

        pt = self.pt.to_dict()

        pt_br = self.pt_br.to_dict()

        sk = self.sk.to_dict()

        sl = self.sl.to_dict()

        zh_hans = self.zh_hans.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "versions": versions,
                "datatypes": datatypes,
                "base": base,
                "cs": cs,
                "da": da,
                "de": de,
                "el": el,
                "en": en,
                "es": es,
                "fr": fr,
                "hu": hu,
                "it": it,
                "nb_NO": nb_no,
                "nl": nl,
                "pl": pl,
                "pt": pt,
                "pt_BR": pt_br,
                "sk": sk,
                "sl": sl,
                "zh_Hans": zh_hans,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.import_open_data_version_meta_data import ImportOpenDataVersionMetaData

        d = dict(src_dict)
        versions = cast(list[str], d.pop("versions"))

        datatypes = cast(list[str], d.pop("datatypes"))

        base = ImportOpenDataVersionMetaData.from_dict(d.pop("base"))

        cs = ImportOpenDataVersionMetaData.from_dict(d.pop("cs"))

        da = ImportOpenDataVersionMetaData.from_dict(d.pop("da"))

        de = ImportOpenDataVersionMetaData.from_dict(d.pop("de"))

        el = ImportOpenDataVersionMetaData.from_dict(d.pop("el"))

        en = ImportOpenDataVersionMetaData.from_dict(d.pop("en"))

        es = ImportOpenDataVersionMetaData.from_dict(d.pop("es"))

        fr = ImportOpenDataVersionMetaData.from_dict(d.pop("fr"))

        hu = ImportOpenDataVersionMetaData.from_dict(d.pop("hu"))

        it = ImportOpenDataVersionMetaData.from_dict(d.pop("it"))

        nb_no = ImportOpenDataVersionMetaData.from_dict(d.pop("nb_NO"))

        nl = ImportOpenDataVersionMetaData.from_dict(d.pop("nl"))

        pl = ImportOpenDataVersionMetaData.from_dict(d.pop("pl"))

        pt = ImportOpenDataVersionMetaData.from_dict(d.pop("pt"))

        pt_br = ImportOpenDataVersionMetaData.from_dict(d.pop("pt_BR"))

        sk = ImportOpenDataVersionMetaData.from_dict(d.pop("sk"))

        sl = ImportOpenDataVersionMetaData.from_dict(d.pop("sl"))

        zh_hans = ImportOpenDataVersionMetaData.from_dict(d.pop("zh_Hans"))

        import_open_data_meta_data = cls(
            versions=versions,
            datatypes=datatypes,
            base=base,
            cs=cs,
            da=da,
            de=de,
            el=el,
            en=en,
            es=es,
            fr=fr,
            hu=hu,
            it=it,
            nb_no=nb_no,
            nl=nl,
            pl=pl,
            pt=pt,
            pt_br=pt_br,
            sk=sk,
            sl=sl,
            zh_hans=zh_hans,
        )

        import_open_data_meta_data.additional_properties = d
        return import_open_data_meta_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.import_open_data_response_detail import ImportOpenDataResponseDetail


T = TypeVar("T", bound="ImportOpenDataResponse")


@_attrs_define
class ImportOpenDataResponse:
    """
    Attributes:
        food (ImportOpenDataResponseDetail | Unset):
        unit (ImportOpenDataResponseDetail | Unset):
        category (ImportOpenDataResponseDetail | Unset):
        property_ (ImportOpenDataResponseDetail | Unset):
        store (ImportOpenDataResponseDetail | Unset):
        conversion (ImportOpenDataResponseDetail | Unset):
    """

    food: ImportOpenDataResponseDetail | Unset = UNSET
    unit: ImportOpenDataResponseDetail | Unset = UNSET
    category: ImportOpenDataResponseDetail | Unset = UNSET
    property_: ImportOpenDataResponseDetail | Unset = UNSET
    store: ImportOpenDataResponseDetail | Unset = UNSET
    conversion: ImportOpenDataResponseDetail | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        food: dict[str, Any] | Unset = UNSET
        if not isinstance(self.food, Unset):
            food = self.food.to_dict()

        unit: dict[str, Any] | Unset = UNSET
        if not isinstance(self.unit, Unset):
            unit = self.unit.to_dict()

        category: dict[str, Any] | Unset = UNSET
        if not isinstance(self.category, Unset):
            category = self.category.to_dict()

        property_: dict[str, Any] | Unset = UNSET
        if not isinstance(self.property_, Unset):
            property_ = self.property_.to_dict()

        store: dict[str, Any] | Unset = UNSET
        if not isinstance(self.store, Unset):
            store = self.store.to_dict()

        conversion: dict[str, Any] | Unset = UNSET
        if not isinstance(self.conversion, Unset):
            conversion = self.conversion.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if food is not UNSET:
            field_dict["food"] = food
        if unit is not UNSET:
            field_dict["unit"] = unit
        if category is not UNSET:
            field_dict["category"] = category
        if property_ is not UNSET:
            field_dict["property"] = property_
        if store is not UNSET:
            field_dict["store"] = store
        if conversion is not UNSET:
            field_dict["conversion"] = conversion

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.import_open_data_response_detail import ImportOpenDataResponseDetail

        d = dict(src_dict)
        _food = d.pop("food", UNSET)
        food: ImportOpenDataResponseDetail | Unset
        if isinstance(_food, Unset):
            food = UNSET
        else:
            food = ImportOpenDataResponseDetail.from_dict(_food)

        _unit = d.pop("unit", UNSET)
        unit: ImportOpenDataResponseDetail | Unset
        if isinstance(_unit, Unset):
            unit = UNSET
        else:
            unit = ImportOpenDataResponseDetail.from_dict(_unit)

        _category = d.pop("category", UNSET)
        category: ImportOpenDataResponseDetail | Unset
        if isinstance(_category, Unset):
            category = UNSET
        else:
            category = ImportOpenDataResponseDetail.from_dict(_category)

        _property_ = d.pop("property", UNSET)
        property_: ImportOpenDataResponseDetail | Unset
        if isinstance(_property_, Unset):
            property_ = UNSET
        else:
            property_ = ImportOpenDataResponseDetail.from_dict(_property_)

        _store = d.pop("store", UNSET)
        store: ImportOpenDataResponseDetail | Unset
        if isinstance(_store, Unset):
            store = UNSET
        else:
            store = ImportOpenDataResponseDetail.from_dict(_store)

        _conversion = d.pop("conversion", UNSET)
        conversion: ImportOpenDataResponseDetail | Unset
        if isinstance(_conversion, Unset):
            conversion = UNSET
        else:
            conversion = ImportOpenDataResponseDetail.from_dict(_conversion)

        import_open_data_response = cls(
            food=food,
            unit=unit,
            category=category,
            property_=property_,
            store=store,
            conversion=conversion,
        )

        import_open_data_response.additional_properties = d
        return import_open_data_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.source_import_duplicate import SourceImportDuplicate
    from ..models.source_import_recipe import SourceImportRecipe


T = TypeVar("T", bound="RecipeFromSourceResponse")


@_attrs_define
class RecipeFromSourceResponse:
    """
    Attributes:
        recipe (SourceImportRecipe | Unset):
        recipe_id (int | Unset):
        images (list[str] | Unset):
        error (bool | Unset):  Default: False.
        msg (str | Unset):  Default: ''.
        duplicates (list[SourceImportDuplicate] | Unset):
    """

    recipe: SourceImportRecipe | Unset = UNSET
    recipe_id: int | Unset = UNSET
    images: list[str] | Unset = UNSET
    error: bool | Unset = False
    msg: str | Unset = ""
    duplicates: list[SourceImportDuplicate] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        recipe: dict[str, Any] | Unset = UNSET
        if not isinstance(self.recipe, Unset):
            recipe = self.recipe.to_dict()

        recipe_id = self.recipe_id

        images: list[str] | Unset = UNSET
        if not isinstance(self.images, Unset):
            images = self.images

        error = self.error

        msg = self.msg

        duplicates: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.duplicates, Unset):
            duplicates = []
            for duplicates_item_data in self.duplicates:
                duplicates_item = duplicates_item_data.to_dict()
                duplicates.append(duplicates_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if recipe is not UNSET:
            field_dict["recipe"] = recipe
        if recipe_id is not UNSET:
            field_dict["recipe_id"] = recipe_id
        if images is not UNSET:
            field_dict["images"] = images
        if error is not UNSET:
            field_dict["error"] = error
        if msg is not UNSET:
            field_dict["msg"] = msg
        if duplicates is not UNSET:
            field_dict["duplicates"] = duplicates

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.source_import_duplicate import SourceImportDuplicate
        from ..models.source_import_recipe import SourceImportRecipe

        d = dict(src_dict)
        _recipe = d.pop("recipe", UNSET)
        recipe: SourceImportRecipe | Unset
        if isinstance(_recipe, Unset):
            recipe = UNSET
        else:
            recipe = SourceImportRecipe.from_dict(_recipe)

        recipe_id = d.pop("recipe_id", UNSET)

        images = cast(list[str], d.pop("images", UNSET))

        error = d.pop("error", UNSET)

        msg = d.pop("msg", UNSET)

        _duplicates = d.pop("duplicates", UNSET)
        duplicates: list[SourceImportDuplicate] | Unset = UNSET
        if _duplicates is not UNSET:
            duplicates = []
            for duplicates_item_data in _duplicates:
                duplicates_item = SourceImportDuplicate.from_dict(duplicates_item_data)

                duplicates.append(duplicates_item)

        recipe_from_source_response = cls(
            recipe=recipe,
            recipe_id=recipe_id,
            images=images,
            error=error,
            msg=msg,
            duplicates=duplicates,
        )

        recipe_from_source_response.additional_properties = d
        return recipe_from_source_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.food import Food
    from ..models.unit import Unit


T = TypeVar("T", bound="Ingredient")


@_attrs_define
class Ingredient:
    """Adds nested create feature

    Attributes:
        food (Food | None):
        unit (None | Unit):
        amount (float):
        conversions (list[Any]):
        used_in_recipes (list[Any]):
        id (int | Unset):
        note (None | str | Unset):
        order (int | Unset):
        is_header (bool | Unset):
        no_amount (bool | Unset):
        original_text (None | str | Unset):
        always_use_plural_unit (bool | Unset):
        always_use_plural_food (bool | Unset):
    """

    food: Food | None
    unit: None | Unit
    amount: float
    conversions: list[Any]
    used_in_recipes: list[Any]
    id: int | Unset = UNSET
    note: None | str | Unset = UNSET
    order: int | Unset = UNSET
    is_header: bool | Unset = UNSET
    no_amount: bool | Unset = UNSET
    original_text: None | str | Unset = UNSET
    always_use_plural_unit: bool | Unset = UNSET
    always_use_plural_food: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.food import Food
        from ..models.unit import Unit

        food: dict[str, Any] | None
        if isinstance(self.food, Food):
            food = self.food.to_dict()
        else:
            food = self.food

        unit: dict[str, Any] | None
        if isinstance(self.unit, Unit):
            unit = self.unit.to_dict()
        else:
            unit = self.unit

        amount = self.amount

        conversions = self.conversions

        used_in_recipes = self.used_in_recipes

        id = self.id

        note: None | str | Unset
        if isinstance(self.note, Unset):
            note = UNSET
        else:
            note = self.note

        order = self.order

        is_header = self.is_header

        no_amount = self.no_amount

        original_text: None | str | Unset
        if isinstance(self.original_text, Unset):
            original_text = UNSET
        else:
            original_text = self.original_text

        always_use_plural_unit = self.always_use_plural_unit

        always_use_plural_food = self.always_use_plural_food

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "food": food,
                "unit": unit,
                "amount": amount,
                "conversions": conversions,
                "used_in_recipes": used_in_recipes,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if note is not UNSET:
            field_dict["note"] = note
        if order is not UNSET:
            field_dict["order"] = order
        if is_header is not UNSET:
            field_dict["is_header"] = is_header
        if no_amount is not UNSET:
            field_dict["no_amount"] = no_amount
        if original_text is not UNSET:
            field_dict["original_text"] = original_text
        if always_use_plural_unit is not UNSET:
            field_dict["always_use_plural_unit"] = always_use_plural_unit
        if always_use_plural_food is not UNSET:
            field_dict["always_use_plural_food"] = always_use_plural_food

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if self.food is None:
            files.append(("food", (None, str(self.food).encode(), "text/plain")))
        else:
            files.append(("food", (None, json.dumps(self.food.to_dict()).encode(), "application/json")))

        if self.unit is None:
            files.append(("unit", (None, str(self.unit).encode(), "text/plain")))
        else:
            files.append(("unit", (None, json.dumps(self.unit.to_dict()).encode(), "application/json")))

        files.append(("amount", (None, str(self.amount).encode(), "text/plain")))

        for conversions_item_element in self.conversions:
            files.append(("conversions", (None, str(conversions_item_element).encode(), "text/plain")))

        for used_in_recipes_item_element in self.used_in_recipes:
            files.append(("used_in_recipes", (None, str(used_in_recipes_item_element).encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.note, Unset):
            if isinstance(self.note, str):
                files.append(("note", (None, str(self.note).encode(), "text/plain")))
            else:
                files.append(("note", (None, str(self.note).encode(), "text/plain")))

        if not isinstance(self.order, Unset):
            files.append(("order", (None, str(self.order).encode(), "text/plain")))

        if not isinstance(self.is_header, Unset):
            files.append(("is_header", (None, str(self.is_header).encode(), "text/plain")))

        if not isinstance(self.no_amount, Unset):
            files.append(("no_amount", (None, str(self.no_amount).encode(), "text/plain")))

        if not isinstance(self.original_text, Unset):
            if isinstance(self.original_text, str):
                files.append(("original_text", (None, str(self.original_text).encode(), "text/plain")))
            else:
                files.append(("original_text", (None, str(self.original_text).encode(), "text/plain")))

        if not isinstance(self.always_use_plural_unit, Unset):
            files.append(("always_use_plural_unit", (None, str(self.always_use_plural_unit).encode(), "text/plain")))

        if not isinstance(self.always_use_plural_food, Unset):
            files.append(("always_use_plural_food", (None, str(self.always_use_plural_food).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.food import Food
        from ..models.unit import Unit

        d = dict(src_dict)

        def _parse_food(data: object) -> Food | None:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                food_type_1 = Food.from_dict(data)

                return food_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(Food | None, data)

        food = _parse_food(d.pop("food"))

        def _parse_unit(data: object) -> None | Unit:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                unit_type_1 = Unit.from_dict(data)

                return unit_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unit, data)

        unit = _parse_unit(d.pop("unit"))

        amount = d.pop("amount")

        conversions = cast(list[Any], d.pop("conversions"))

        used_in_recipes = cast(list[Any], d.pop("used_in_recipes"))

        id = d.pop("id", UNSET)

        def _parse_note(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        note = _parse_note(d.pop("note", UNSET))

        order = d.pop("order", UNSET)

        is_header = d.pop("is_header", UNSET)

        no_amount = d.pop("no_amount", UNSET)

        def _parse_original_text(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        original_text = _parse_original_text(d.pop("original_text", UNSET))

        always_use_plural_unit = d.pop("always_use_plural_unit", UNSET)

        always_use_plural_food = d.pop("always_use_plural_food", UNSET)

        ingredient = cls(
            food=food,
            unit=unit,
            amount=amount,
            conversions=conversions,
            used_in_recipes=used_in_recipes,
            id=id,
            note=note,
            order=order,
            is_header=is_header,
            no_amount=no_amount,
            original_text=original_text,
            always_use_plural_unit=always_use_plural_unit,
            always_use_plural_food=always_use_plural_food,
        )

        ingredient.additional_properties = d
        return ingredient

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types

if TYPE_CHECKING:
    from ..models.shopping_list_entry_simple_create import ShoppingListEntrySimpleCreate


T = TypeVar("T", bound="ShoppingListEntryBulkCreate")


@_attrs_define
class ShoppingListEntryBulkCreate:
    """
    Attributes:
        entries (list[ShoppingListEntrySimpleCreate]):
    """

    entries: list[ShoppingListEntrySimpleCreate]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        entries = []
        for entries_item_data in self.entries:
            entries_item = entries_item_data.to_dict()
            entries.append(entries_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "entries": entries,
            }
        )

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        for entries_item_element in self.entries:
            files.append(("entries", (None, json.dumps(entries_item_element.to_dict()).encode(), "application/json")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.shopping_list_entry_simple_create import ShoppingListEntrySimpleCreate

        d = dict(src_dict)
        entries = []
        _entries = d.pop("entries")
        for entries_item_data in _entries:
            entries_item = ShoppingListEntrySimpleCreate.from_dict(entries_item_data)

            entries.append(entries_item)

        shopping_list_entry_bulk_create = cls(
            entries=entries,
        )

        shopping_list_entry_bulk_create.additional_properties = d
        return shopping_list_entry_bulk_create

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user import User


T = TypeVar("T", bound="UserFileView")


@_attrs_define
class UserFileView:
    """
    Attributes:
        name (str):
        file_download (str):
        preview (str):
        file_size_kb (int):
        created_by (User): Adds nested create feature
        created_at (datetime.datetime):
        id (int | Unset):
    """

    name: str
    file_download: str
    preview: str
    file_size_kb: int
    created_by: User
    created_at: datetime.datetime
    id: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        file_download = self.file_download

        preview = self.preview

        file_size_kb = self.file_size_kb

        created_by = self.created_by.to_dict()

        created_at = self.created_at.isoformat()

        id = self.id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "file_download": file_download,
                "preview": preview,
                "file_size_kb": file_size_kb,
                "created_by": created_by,
                "created_at": created_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user import User

        d = dict(src_dict)
        name = d.pop("name")

        file_download = d.pop("file_download")

        preview = d.pop("preview")

        file_size_kb = d.pop("file_size_kb")

        created_by = User.from_dict(d.pop("created_by"))

        created_at = isoparse(d.pop("created_at"))

        id = d.pop("id", UNSET)

        user_file_view = cls(
            name=name,
            file_download=file_download,
            preview=preview,
            file_size_kb=file_size_kb,
            created_by=created_by,
            created_at=created_at,
            id=id,
        )

        user_file_view.additional_properties = d
        return user_file_view

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="RecipeImage")


@_attrs_define
class RecipeImage:
    """Adds nested create feature

    Attributes:
        image (None | str | Unset):
        image_url (None | str | Unset):
    """

    image: None | str | Unset = UNSET
    image_url: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        image: None | str | Unset
        if isinstance(self.image, Unset):
            image = UNSET
        else:
            image = self.image

        image_url: None | str | Unset
        if isinstance(self.image_url, Unset):
            image_url = UNSET
        else:
            image_url = self.image_url

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if image is not UNSET:
            field_dict["image"] = image
        if image_url is not UNSET:
            field_dict["image_url"] = image_url

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.image, Unset):
            if isinstance(self.image, str):
                files.append(("image", (None, str(self.image).encode(), "text/plain")))
            else:
                files.append(("image", (None, str(self.image).encode(), "text/plain")))

        if not isinstance(self.image_url, Unset):
            if isinstance(self.image_url, str):
                files.append(("image_url", (None, str(self.image_url).encode(), "text/plain")))
            else:
                files.append(("image_url", (None, str(self.image_url).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_image(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        image = _parse_image(d.pop("image", UNSET))

        def _parse_image_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        image_url = _parse_image_url(d.pop("image_url", UNSET))

        recipe_image = cls(
            image=image,
            image_url=image_url,
        )

        recipe_image.additional_properties = d
        return recipe_image

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="Keyword")


@_attrs_define
class Keyword:
    """Moves `UniqueValidator`'s from the validation stage to the save stage.
    It solves the problem with nested validation for unique fields on update.

    If you want more details, you can read related issues and articles:
    https://github.com/beda-software/drf-writable-nested/issues/1
    http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

    Example of usage:
    ```
        class Child(models.Model):
        field = models.CharField(unique=True)


    class Parent(models.Model):
        child = models.ForeignKey('Child')


    class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
        class Meta:
            model = Child


    class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
        child = ChildSerializer()

        class Meta:
            model = Parent
    ```

    Note: `UniqueFieldsMixin` must be applied only on the serializer
    which has unique fields.

    Note: When you are using both mixins
    (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
    you should put `UniqueFieldsMixin` ahead.

        Attributes:
            name (str):
            label (str):
            parent (int):
            numchild (int):
            created_at (datetime.datetime):
            updated_at (datetime.datetime):
            full_name (str): Returns a string representation of a tree node and it's ancestors,
                e.g. 'Cuisine > Asian > Chinese > Catonese'.
            id (int | Unset):
            description (str | Unset):
    """

    name: str
    label: str
    parent: int
    numchild: int
    created_at: datetime.datetime
    updated_at: datetime.datetime
    full_name: str
    id: int | Unset = UNSET
    description: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        label = self.label

        parent = self.parent

        numchild = self.numchild

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        full_name = self.full_name

        id = self.id

        description = self.description

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "label": label,
                "parent": parent,
                "numchild": numchild,
                "created_at": created_at,
                "updated_at": updated_at,
                "full_name": full_name,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("name", (None, str(self.name).encode(), "text/plain")))

        files.append(("label", (None, str(self.label).encode(), "text/plain")))

        files.append(("parent", (None, str(self.parent).encode(), "text/plain")))

        files.append(("numchild", (None, str(self.numchild).encode(), "text/plain")))

        files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        files.append(("updated_at", (None, self.updated_at.isoformat().encode(), "text/plain")))

        files.append(("full_name", (None, str(self.full_name).encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.description, Unset):
            files.append(("description", (None, str(self.description).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        label = d.pop("label")

        parent = d.pop("parent")

        numchild = d.pop("numchild")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        full_name = d.pop("full_name")

        id = d.pop("id", UNSET)

        description = d.pop("description", UNSET)

        keyword = cls(
            name=name,
            label=label,
            parent=parent,
            numchild=numchild,
            created_at=created_at,
            updated_at=updated_at,
            full_name=full_name,
            id=id,
            description=description,
        )

        keyword.additional_properties = d
        return keyword

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="PatchedKeyword")


@_attrs_define
class PatchedKeyword:
    """Moves `UniqueValidator`'s from the validation stage to the save stage.
    It solves the problem with nested validation for unique fields on update.

    If you want more details, you can read related issues and articles:
    https://github.com/beda-software/drf-writable-nested/issues/1
    http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

    Example of usage:
    ```
        class Child(models.Model):
        field = models.CharField(unique=True)


    class Parent(models.Model):
        child = models.ForeignKey('Child')


    class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
        class Meta:
            model = Child


    class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
        child = ChildSerializer()

        class Meta:
            model = Parent
    ```

    Note: `UniqueFieldsMixin` must be applied only on the serializer
    which has unique fields.

    Note: When you are using both mixins
    (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
    you should put `UniqueFieldsMixin` ahead.

        Attributes:
            id (int | Unset):
            name (str | Unset):
            label (str | Unset):
            description (str | Unset):
            parent (int | Unset):
            numchild (int | Unset):
            created_at (datetime.datetime | Unset):
            updated_at (datetime.datetime | Unset):
            full_name (str | Unset): Returns a string representation of a tree node and it's ancestors,
                e.g. 'Cuisine > Asian > Chinese > Catonese'.
    """

    id: int | Unset = UNSET
    name: str | Unset = UNSET
    label: str | Unset = UNSET
    description: str | Unset = UNSET
    parent: int | Unset = UNSET
    numchild: int | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    updated_at: datetime.datetime | Unset = UNSET
    full_name: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        label = self.label

        description = self.description

        parent = self.parent

        numchild = self.numchild

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: str | Unset = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        full_name = self.full_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if label is not UNSET:
            field_dict["label"] = label
        if description is not UNSET:
            field_dict["description"] = description
        if parent is not UNSET:
            field_dict["parent"] = parent
        if numchild is not UNSET:
            field_dict["numchild"] = numchild
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if full_name is not UNSET:
            field_dict["full_name"] = full_name

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.name, Unset):
            files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.label, Unset):
            files.append(("label", (None, str(self.label).encode(), "text/plain")))

        if not isinstance(self.description, Unset):
            files.append(("description", (None, str(self.description).encode(), "text/plain")))

        if not isinstance(self.parent, Unset):
            files.append(("parent", (None, str(self.parent).encode(), "text/plain")))

        if not isinstance(self.numchild, Unset):
            files.append(("numchild", (None, str(self.numchild).encode(), "text/plain")))

        if not isinstance(self.created_at, Unset):
            files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.updated_at, Unset):
            files.append(("updated_at", (None, self.updated_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.full_name, Unset):
            files.append(("full_name", (None, str(self.full_name).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        label = d.pop("label", UNSET)

        description = d.pop("description", UNSET)

        parent = d.pop("parent", UNSET)

        numchild = d.pop("numchild", UNSET)

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: datetime.datetime | Unset
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        full_name = d.pop("full_name", UNSET)

        patched_keyword = cls(
            id=id,
            name=name,
            label=label,
            description=description,
            parent=parent,
            numchild=numchild,
            created_at=created_at,
            updated_at=updated_at,
            full_name=full_name,
        )

        patched_keyword.additional_properties = d
        return patched_keyword

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.recipe_book import RecipeBook
    from ..models.recipe_overview import RecipeOverview


T = TypeVar("T", bound="PatchedRecipeBookEntry")


@_attrs_define
class PatchedRecipeBookEntry:
    """
    Attributes:
        id (int | Unset):
        book (int | Unset):
        book_content (RecipeBook | Unset): Adds nested create feature
        recipe (int | Unset):
        recipe_content (RecipeOverview | Unset): Adds nested create feature
    """

    id: int | Unset = UNSET
    book: int | Unset = UNSET
    book_content: RecipeBook | Unset = UNSET
    recipe: int | Unset = UNSET
    recipe_content: RecipeOverview | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        book = self.book

        book_content: dict[str, Any] | Unset = UNSET
        if not isinstance(self.book_content, Unset):
            book_content = self.book_content.to_dict()

        recipe = self.recipe

        recipe_content: dict[str, Any] | Unset = UNSET
        if not isinstance(self.recipe_content, Unset):
            recipe_content = self.recipe_content.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if book is not UNSET:
            field_dict["book"] = book
        if book_content is not UNSET:
            field_dict["book_content"] = book_content
        if recipe is not UNSET:
            field_dict["recipe"] = recipe
        if recipe_content is not UNSET:
            field_dict["recipe_content"] = recipe_content

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.book, Unset):
            files.append(("book", (None, str(self.book).encode(), "text/plain")))

        if not isinstance(self.book_content, Unset):
            files.append(("book_content", (None, json.dumps(self.book_content.to_dict()).encode(), "application/json")))

        if not isinstance(self.recipe, Unset):
            files.append(("recipe", (None, str(self.recipe).encode(), "text/plain")))

        if not isinstance(self.recipe_content, Unset):
            files.append(
                ("recipe_content", (None, json.dumps(self.recipe_content.to_dict()).encode(), "application/json"))
            )

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.recipe_book import RecipeBook
        from ..models.recipe_overview import RecipeOverview

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        book = d.pop("book", UNSET)

        _book_content = d.pop("book_content", UNSET)
        book_content: RecipeBook | Unset
        if isinstance(_book_content, Unset):
            book_content = UNSET
        else:
            book_content = RecipeBook.from_dict(_book_content)

        recipe = d.pop("recipe", UNSET)

        _recipe_content = d.pop("recipe_content", UNSET)
        recipe_content: RecipeOverview | Unset
        if isinstance(_recipe_content, Unset):
            recipe_content = UNSET
        else:
            recipe_content = RecipeOverview.from_dict(_recipe_content)

        patched_recipe_book_entry = cls(
            id=id,
            book=book,
            book_content=book_content,
            recipe=recipe,
            recipe_content=recipe_content,
        )

        patched_recipe_book_entry.additional_properties = d
        return patched_recipe_book_entry

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any
from urllib.parse import quote

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.paginated_generic_model_reference_list import PaginatedGenericModelReferenceList
from ...types import UNSET, Response, Unset


def _get_kwargs(
    id: int,
    *,
    cache: bool | Unset = True,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    params["cache"] = cache

    params["page"] = page

    params["page_size"] = page_size

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/sync/{id}/protecting/".format(
            id=quote(str(id), safe=""),
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> PaginatedGenericModelReferenceList | None:
    if response.status_code == 200:
        response_200 = PaginatedGenericModelReferenceList.from_dict(response.json())

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[PaginatedGenericModelReferenceList]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: int,
    *,
    client: AuthenticatedClient,
    cache: bool | Unset = True,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
) -> Response[PaginatedGenericModelReferenceList]:
    """get a paginated list of objects that are protecting the selected object form being deleted

    Args:
        id (int):
        cache (bool | Unset):  Default: True.
        page (int | Unset):
        page_size (int | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedGenericModelReferenceList]
    """

    kwargs = _get_kwargs(
        id=id,
        cache=cache,
        page=page,
        page_size=page_size,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: int,
    *,
    client: AuthenticatedClient,
    cache: bool | Unset = True,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
) -> PaginatedGenericModelReferenceList | None:
    """get a paginated list of objects that are protecting the selected object form being deleted

    Args:
        id (int):
        cache (bool | Unset):  Default: True.
        page (int | Unset):
        page_size (int | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedGenericModelReferenceList
    """

    return sync_detailed(
        id=id,
        client=client,
        cache=cache,
        page=page,
        page_size=page_size,
    ).parsed


async def asyncio_detailed(
    id: int,
    *,
    client: AuthenticatedClient,
    cache: bool | Unset = True,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
) -> Response[PaginatedGenericModelReferenceList]:
    """get a paginated list of objects that are protecting the selected object form being deleted

    Args:
        id (int):
        cache (bool | Unset):  Default: True.
        page (int | Unset):
        page_size (int | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedGenericModelReferenceList]
    """

    kwargs = _get_kwargs(
        id=id,
        cache=cache,
        page=page,
        page_size=page_size,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: int,
    *,
    client: AuthenticatedClient,
    cache: bool | Unset = True,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
) -> PaginatedGenericModelReferenceList | None:
    """get a paginated list of objects that are protecting the selected object form being deleted

    Args:
        id (int):
        cache (bool | Unset):  Default: True.
        page (int | Unset):
        page_size (int | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedGenericModelReferenceList
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            cache=cache,
            page=page,
            page_size=page_size,
        )
    ).parsed



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.ai_provider import AiProvider


T = TypeVar("T", bound="AiLog")


@_attrs_define
class AiLog:
    """
    Attributes:
        ai_provider (AiProvider):
        function (str):
        credit_cost (float):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        id (int | Unset):
        credits_from_balance (bool | Unset):
        input_tokens (int | Unset):
        output_tokens (int | Unset):
        start_time (datetime.datetime | None | Unset):
        end_time (datetime.datetime | None | Unset):
        created_by (int | None | Unset):
    """

    ai_provider: AiProvider
    function: str
    credit_cost: float
    created_at: datetime.datetime
    updated_at: datetime.datetime
    id: int | Unset = UNSET
    credits_from_balance: bool | Unset = UNSET
    input_tokens: int | Unset = UNSET
    output_tokens: int | Unset = UNSET
    start_time: datetime.datetime | None | Unset = UNSET
    end_time: datetime.datetime | None | Unset = UNSET
    created_by: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        ai_provider = self.ai_provider.to_dict()

        function = self.function

        credit_cost = self.credit_cost

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        id = self.id

        credits_from_balance = self.credits_from_balance

        input_tokens = self.input_tokens

        output_tokens = self.output_tokens

        start_time: None | str | Unset
        if isinstance(self.start_time, Unset):
            start_time = UNSET
        elif isinstance(self.start_time, datetime.datetime):
            start_time = self.start_time.isoformat()
        else:
            start_time = self.start_time

        end_time: None | str | Unset
        if isinstance(self.end_time, Unset):
            end_time = UNSET
        elif isinstance(self.end_time, datetime.datetime):
            end_time = self.end_time.isoformat()
        else:
            end_time = self.end_time

        created_by: int | None | Unset
        if isinstance(self.created_by, Unset):
            created_by = UNSET
        else:
            created_by = self.created_by

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "ai_provider": ai_provider,
                "function": function,
                "credit_cost": credit_cost,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if credits_from_balance is not UNSET:
            field_dict["credits_from_balance"] = credits_from_balance
        if input_tokens is not UNSET:
            field_dict["input_tokens"] = input_tokens
        if output_tokens is not UNSET:
            field_dict["output_tokens"] = output_tokens
        if start_time is not UNSET:
            field_dict["start_time"] = start_time
        if end_time is not UNSET:
            field_dict["end_time"] = end_time
        if created_by is not UNSET:
            field_dict["created_by"] = created_by

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.ai_provider import AiProvider

        d = dict(src_dict)
        ai_provider = AiProvider.from_dict(d.pop("ai_provider"))

        function = d.pop("function")

        credit_cost = d.pop("credit_cost")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        id = d.pop("id", UNSET)

        credits_from_balance = d.pop("credits_from_balance", UNSET)

        input_tokens = d.pop("input_tokens", UNSET)

        output_tokens = d.pop("output_tokens", UNSET)

        def _parse_start_time(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                start_time_type_0 = isoparse(data)

                return start_time_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        start_time = _parse_start_time(d.pop("start_time", UNSET))

        def _parse_end_time(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                end_time_type_0 = isoparse(data)

                return end_time_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        end_time = _parse_end_time(d.pop("end_time", UNSET))

        def _parse_created_by(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        created_by = _parse_created_by(d.pop("created_by", UNSET))

        ai_log = cls(
            ai_provider=ai_provider,
            function=function,
            credit_cost=credit_cost,
            created_at=created_at,
            updated_at=updated_at,
            id=id,
            credits_from_balance=credits_from_balance,
            input_tokens=input_tokens,
            output_tokens=output_tokens,
            start_time=start_time,
            end_time=end_time,
            created_by=created_by,
        )

        ai_log.additional_properties = d
        return ai_log

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..models.connector_config_type_enum import ConnectorConfigTypeEnum
from ..types import UNSET, Unset

T = TypeVar("T", bound="ConnectorConfig")


@_attrs_define
class ConnectorConfig:
    """
    Attributes:
        name (str):
        created_by (int):
        id (int | Unset):
        type_ (ConnectorConfigTypeEnum | Unset): * `HomeAssistant` - HomeAssistant
        url (None | str | Unset):
        token (None | str | Unset):
        todo_entity (None | str | Unset):
        enabled (bool | Unset): Is Connector Enabled
        on_shopping_list_entry_created_enabled (bool | Unset):
        on_shopping_list_entry_updated_enabled (bool | Unset):
        on_shopping_list_entry_deleted_enabled (bool | Unset):
        supports_description_field (bool | Unset): Does the todo entity support the description field
    """

    name: str
    created_by: int
    id: int | Unset = UNSET
    type_: ConnectorConfigTypeEnum | Unset = UNSET
    url: None | str | Unset = UNSET
    token: None | str | Unset = UNSET
    todo_entity: None | str | Unset = UNSET
    enabled: bool | Unset = UNSET
    on_shopping_list_entry_created_enabled: bool | Unset = UNSET
    on_shopping_list_entry_updated_enabled: bool | Unset = UNSET
    on_shopping_list_entry_deleted_enabled: bool | Unset = UNSET
    supports_description_field: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        created_by = self.created_by

        id = self.id

        type_: str | Unset = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        url: None | str | Unset
        if isinstance(self.url, Unset):
            url = UNSET
        else:
            url = self.url

        token: None | str | Unset
        if isinstance(self.token, Unset):
            token = UNSET
        else:
            token = self.token

        todo_entity: None | str | Unset
        if isinstance(self.todo_entity, Unset):
            todo_entity = UNSET
        else:
            todo_entity = self.todo_entity

        enabled = self.enabled

        on_shopping_list_entry_created_enabled = self.on_shopping_list_entry_created_enabled

        on_shopping_list_entry_updated_enabled = self.on_shopping_list_entry_updated_enabled

        on_shopping_list_entry_deleted_enabled = self.on_shopping_list_entry_deleted_enabled

        supports_description_field = self.supports_description_field

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "created_by": created_by,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if type_ is not UNSET:
            field_dict["type"] = type_
        if url is not UNSET:
            field_dict["url"] = url
        if token is not UNSET:
            field_dict["token"] = token
        if todo_entity is not UNSET:
            field_dict["todo_entity"] = todo_entity
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if on_shopping_list_entry_created_enabled is not UNSET:
            field_dict["on_shopping_list_entry_created_enabled"] = on_shopping_list_entry_created_enabled
        if on_shopping_list_entry_updated_enabled is not UNSET:
            field_dict["on_shopping_list_entry_updated_enabled"] = on_shopping_list_entry_updated_enabled
        if on_shopping_list_entry_deleted_enabled is not UNSET:
            field_dict["on_shopping_list_entry_deleted_enabled"] = on_shopping_list_entry_deleted_enabled
        if supports_description_field is not UNSET:
            field_dict["supports_description_field"] = supports_description_field

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("name", (None, str(self.name).encode(), "text/plain")))

        files.append(("created_by", (None, str(self.created_by).encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.type_, Unset):
            files.append(("type", (None, str(self.type_.value).encode(), "text/plain")))

        if not isinstance(self.url, Unset):
            if isinstance(self.url, str):
                files.append(("url", (None, str(self.url).encode(), "text/plain")))
            else:
                files.append(("url", (None, str(self.url).encode(), "text/plain")))

        if not isinstance(self.token, Unset):
            if isinstance(self.token, str):
                files.append(("token", (None, str(self.token).encode(), "text/plain")))
            else:
                files.append(("token", (None, str(self.token).encode(), "text/plain")))

        if not isinstance(self.todo_entity, Unset):
            if isinstance(self.todo_entity, str):
                files.append(("todo_entity", (None, str(self.todo_entity).encode(), "text/plain")))
            else:
                files.append(("todo_entity", (None, str(self.todo_entity).encode(), "text/plain")))

        if not isinstance(self.enabled, Unset):
            files.append(("enabled", (None, str(self.enabled).encode(), "text/plain")))

        if not isinstance(self.on_shopping_list_entry_created_enabled, Unset):
            files.append(
                (
                    "on_shopping_list_entry_created_enabled",
                    (None, str(self.on_shopping_list_entry_created_enabled).encode(), "text/plain"),
                )
            )

        if not isinstance(self.on_shopping_list_entry_updated_enabled, Unset):
            files.append(
                (
                    "on_shopping_list_entry_updated_enabled",
                    (None, str(self.on_shopping_list_entry_updated_enabled).encode(), "text/plain"),
                )
            )

        if not isinstance(self.on_shopping_list_entry_deleted_enabled, Unset):
            files.append(
                (
                    "on_shopping_list_entry_deleted_enabled",
                    (None, str(self.on_shopping_list_entry_deleted_enabled).encode(), "text/plain"),
                )
            )

        if not isinstance(self.supports_description_field, Unset):
            files.append(
                ("supports_description_field", (None, str(self.supports_description_field).encode(), "text/plain"))
            )

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        created_by = d.pop("created_by")

        id = d.pop("id", UNSET)

        _type_ = d.pop("type", UNSET)
        type_: ConnectorConfigTypeEnum | Unset
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = ConnectorConfigTypeEnum(_type_)

        def _parse_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        url = _parse_url(d.pop("url", UNSET))

        def _parse_token(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        token = _parse_token(d.pop("token", UNSET))

        def _parse_todo_entity(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        todo_entity = _parse_todo_entity(d.pop("todo_entity", UNSET))

        enabled = d.pop("enabled", UNSET)

        on_shopping_list_entry_created_enabled = d.pop("on_shopping_list_entry_created_enabled", UNSET)

        on_shopping_list_entry_updated_enabled = d.pop("on_shopping_list_entry_updated_enabled", UNSET)

        on_shopping_list_entry_deleted_enabled = d.pop("on_shopping_list_entry_deleted_enabled", UNSET)

        supports_description_field = d.pop("supports_description_field", UNSET)

        connector_config = cls(
            name=name,
            created_by=created_by,
            id=id,
            type_=type_,
            url=url,
            token=token,
            todo_entity=todo_entity,
            enabled=enabled,
            on_shopping_list_entry_created_enabled=on_shopping_list_entry_created_enabled,
            on_shopping_list_entry_updated_enabled=on_shopping_list_entry_updated_enabled,
            on_shopping_list_entry_deleted_enabled=on_shopping_list_entry_deleted_enabled,
            supports_description_field=supports_description_field,
        )

        connector_config.additional_properties = d
        return connector_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.source_import_property_type import SourceImportPropertyType


T = TypeVar("T", bound="SourceImportProperty")


@_attrs_define
class SourceImportProperty:
    """
    Attributes:
        property_type (SourceImportPropertyType):
        property_amount (float):
    """

    property_type: SourceImportPropertyType
    property_amount: float
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        property_type = self.property_type.to_dict()

        property_amount = self.property_amount

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "property_type": property_type,
                "property_amount": property_amount,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.source_import_property_type import SourceImportPropertyType

        d = dict(src_dict)
        property_type = SourceImportPropertyType.from_dict(d.pop("property_type"))

        property_amount = d.pop("property_amount")

        source_import_property = cls(
            property_type=property_type,
            property_amount=property_amount,
        )

        source_import_property.additional_properties = d
        return source_import_property

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.paginated_recipe_overview_list import PaginatedRecipeOverviewList
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    books: list[int] | Unset = UNSET,
    books_and: list[int] | Unset = UNSET,
    books_and_not: list[int] | Unset = UNSET,
    books_or: list[int] | Unset = UNSET,
    books_or_not: list[int] | Unset = UNSET,
    cookedon_gte: datetime.date | Unset = UNSET,
    cookedon_lte: datetime.date | Unset = UNSET,
    createdby: int | Unset = UNSET,
    createdon: datetime.date | Unset = UNSET,
    createdon_gte: datetime.date | Unset = UNSET,
    createdon_lte: datetime.date | Unset = UNSET,
    filter_: int | Unset = UNSET,
    foods: list[int] | Unset = UNSET,
    foods_and: list[int] | Unset = UNSET,
    foods_and_not: list[int] | Unset = UNSET,
    foods_or: list[int] | Unset = UNSET,
    foods_or_not: list[int] | Unset = UNSET,
    internal: bool | Unset = UNSET,
    keywords: list[int] | Unset = UNSET,
    keywords_and: list[int] | Unset = UNSET,
    keywords_and_not: list[int] | Unset = UNSET,
    keywords_or: list[int] | Unset = UNSET,
    keywords_or_not: list[int] | Unset = UNSET,
    makenow: bool | Unset = UNSET,
    new: bool | Unset = UNSET,
    num_recent: int | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    query: str | Unset = UNSET,
    random: bool | Unset = UNSET,
    rating: int | Unset = UNSET,
    rating_gte: int | Unset = UNSET,
    rating_lte: int | Unset = UNSET,
    sort_order: str | Unset = UNSET,
    timescooked: int | Unset = UNSET,
    timescooked_gte: int | Unset = UNSET,
    timescooked_lte: int | Unset = UNSET,
    units: int | Unset = UNSET,
    updatedon: datetime.date | Unset = UNSET,
    updatedon_gte: datetime.date | Unset = UNSET,
    updatedon_lte: datetime.date | Unset = UNSET,
    viewedon_gte: datetime.date | Unset = UNSET,
    viewedon_lte: datetime.date | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    json_books: list[int] | Unset = UNSET
    if not isinstance(books, Unset):
        json_books = books

    params["books"] = json_books

    json_books_and: list[int] | Unset = UNSET
    if not isinstance(books_and, Unset):
        json_books_and = books_and

    params["books_and"] = json_books_and

    json_books_and_not: list[int] | Unset = UNSET
    if not isinstance(books_and_not, Unset):
        json_books_and_not = books_and_not

    params["books_and_not"] = json_books_and_not

    json_books_or: list[int] | Unset = UNSET
    if not isinstance(books_or, Unset):
        json_books_or = books_or

    params["books_or"] = json_books_or

    json_books_or_not: list[int] | Unset = UNSET
    if not isinstance(books_or_not, Unset):
        json_books_or_not = books_or_not

    params["books_or_not"] = json_books_or_not

    json_cookedon_gte: str | Unset = UNSET
    if not isinstance(cookedon_gte, Unset):
        json_cookedon_gte = cookedon_gte.isoformat()
    params["cookedon_gte"] = json_cookedon_gte

    json_cookedon_lte: str | Unset = UNSET
    if not isinstance(cookedon_lte, Unset):
        json_cookedon_lte = cookedon_lte.isoformat()
    params["cookedon_lte"] = json_cookedon_lte

    params["createdby"] = createdby

    json_createdon: str | Unset = UNSET
    if not isinstance(createdon, Unset):
        json_createdon = createdon.isoformat()
    params["createdon"] = json_createdon

    json_createdon_gte: str | Unset = UNSET
    if not isinstance(createdon_gte, Unset):
        json_createdon_gte = createdon_gte.isoformat()
    params["createdon_gte"] = json_createdon_gte

    json_createdon_lte: str | Unset = UNSET
    if not isinstance(createdon_lte, Unset):
        json_createdon_lte = createdon_lte.isoformat()
    params["createdon_lte"] = json_createdon_lte

    params["filter"] = filter_

    json_foods: list[int] | Unset = UNSET
    if not isinstance(foods, Unset):
        json_foods = foods

    params["foods"] = json_foods

    json_foods_and: list[int] | Unset = UNSET
    if not isinstance(foods_and, Unset):
        json_foods_and = foods_and

    params["foods_and"] = json_foods_and

    json_foods_and_not: list[int] | Unset = UNSET
    if not isinstance(foods_and_not, Unset):
        json_foods_and_not = foods_and_not

    params["foods_and_not"] = json_foods_and_not

    json_foods_or: list[int] | Unset = UNSET
    if not isinstance(foods_or, Unset):
        json_foods_or = foods_or

    params["foods_or"] = json_foods_or

    json_foods_or_not: list[int] | Unset = UNSET
    if not isinstance(foods_or_not, Unset):
        json_foods_or_not = foods_or_not

    params["foods_or_not"] = json_foods_or_not

    params["internal"] = internal

    json_keywords: list[int] | Unset = UNSET
    if not isinstance(keywords, Unset):
        json_keywords = keywords

    params["keywords"] = json_keywords

    json_keywords_and: list[int] | Unset = UNSET
    if not isinstance(keywords_and, Unset):
        json_keywords_and = keywords_and

    params["keywords_and"] = json_keywords_and

    json_keywords_and_not: list[int] | Unset = UNSET
    if not isinstance(keywords_and_not, Unset):
        json_keywords_and_not = keywords_and_not

    params["keywords_and_not"] = json_keywords_and_not

    json_keywords_or: list[int] | Unset = UNSET
    if not isinstance(keywords_or, Unset):
        json_keywords_or = keywords_or

    params["keywords_or"] = json_keywords_or

    json_keywords_or_not: list[int] | Unset = UNSET
    if not isinstance(keywords_or_not, Unset):
        json_keywords_or_not = keywords_or_not

    params["keywords_or_not"] = json_keywords_or_not

    params["makenow"] = makenow

    params["new"] = new

    params["num_recent"] = num_recent

    params["page"] = page

    params["page_size"] = page_size

    params["query"] = query

    params["random"] = random

    params["rating"] = rating

    params["rating_gte"] = rating_gte

    params["rating_lte"] = rating_lte

    params["sort_order"] = sort_order

    params["timescooked"] = timescooked

    params["timescooked_gte"] = timescooked_gte

    params["timescooked_lte"] = timescooked_lte

    params["units"] = units

    json_updatedon: str | Unset = UNSET
    if not isinstance(updatedon, Unset):
        json_updatedon = updatedon.isoformat()
    params["updatedon"] = json_updatedon

    json_updatedon_gte: str | Unset = UNSET
    if not isinstance(updatedon_gte, Unset):
        json_updatedon_gte = updatedon_gte.isoformat()
    params["updatedon_gte"] = json_updatedon_gte

    json_updatedon_lte: str | Unset = UNSET
    if not isinstance(updatedon_lte, Unset):
        json_updatedon_lte = updatedon_lte.isoformat()
    params["updatedon_lte"] = json_updatedon_lte

    json_viewedon_gte: str | Unset = UNSET
    if not isinstance(viewedon_gte, Unset):
        json_viewedon_gte = viewedon_gte.isoformat()
    params["viewedon_gte"] = json_viewedon_gte

    json_viewedon_lte: str | Unset = UNSET
    if not isinstance(viewedon_lte, Unset):
        json_viewedon_lte = viewedon_lte.isoformat()
    params["viewedon_lte"] = json_viewedon_lte

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/recipe/",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> PaginatedRecipeOverviewList | None:
    if response.status_code == 200:
        response_200 = PaginatedRecipeOverviewList.from_dict(response.json())

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[PaginatedRecipeOverviewList]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    books: list[int] | Unset = UNSET,
    books_and: list[int] | Unset = UNSET,
    books_and_not: list[int] | Unset = UNSET,
    books_or: list[int] | Unset = UNSET,
    books_or_not: list[int] | Unset = UNSET,
    cookedon_gte: datetime.date | Unset = UNSET,
    cookedon_lte: datetime.date | Unset = UNSET,
    createdby: int | Unset = UNSET,
    createdon: datetime.date | Unset = UNSET,
    createdon_gte: datetime.date | Unset = UNSET,
    createdon_lte: datetime.date | Unset = UNSET,
    filter_: int | Unset = UNSET,
    foods: list[int] | Unset = UNSET,
    foods_and: list[int] | Unset = UNSET,
    foods_and_not: list[int] | Unset = UNSET,
    foods_or: list[int] | Unset = UNSET,
    foods_or_not: list[int] | Unset = UNSET,
    internal: bool | Unset = UNSET,
    keywords: list[int] | Unset = UNSET,
    keywords_and: list[int] | Unset = UNSET,
    keywords_and_not: list[int] | Unset = UNSET,
    keywords_or: list[int] | Unset = UNSET,
    keywords_or_not: list[int] | Unset = UNSET,
    makenow: bool | Unset = UNSET,
    new: bool | Unset = UNSET,
    num_recent: int | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    query: str | Unset = UNSET,
    random: bool | Unset = UNSET,
    rating: int | Unset = UNSET,
    rating_gte: int | Unset = UNSET,
    rating_lte: int | Unset = UNSET,
    sort_order: str | Unset = UNSET,
    timescooked: int | Unset = UNSET,
    timescooked_gte: int | Unset = UNSET,
    timescooked_lte: int | Unset = UNSET,
    units: int | Unset = UNSET,
    updatedon: datetime.date | Unset = UNSET,
    updatedon_gte: datetime.date | Unset = UNSET,
    updatedon_lte: datetime.date | Unset = UNSET,
    viewedon_gte: datetime.date | Unset = UNSET,
    viewedon_lte: datetime.date | Unset = UNSET,
) -> Response[PaginatedRecipeOverviewList]:
    """logs request counts to redis cache total/per user/

    Args:
        books (list[int] | Unset):
        books_and (list[int] | Unset):
        books_and_not (list[int] | Unset):
        books_or (list[int] | Unset):
        books_or_not (list[int] | Unset):
        cookedon_gte (datetime.date | Unset):
        cookedon_lte (datetime.date | Unset):
        createdby (int | Unset):
        createdon (datetime.date | Unset):
        createdon_gte (datetime.date | Unset):
        createdon_lte (datetime.date | Unset):
        filter_ (int | Unset):
        foods (list[int] | Unset):
        foods_and (list[int] | Unset):
        foods_and_not (list[int] | Unset):
        foods_or (list[int] | Unset):
        foods_or_not (list[int] | Unset):
        internal (bool | Unset):
        keywords (list[int] | Unset):
        keywords_and (list[int] | Unset):
        keywords_and_not (list[int] | Unset):
        keywords_or (list[int] | Unset):
        keywords_or_not (list[int] | Unset):
        makenow (bool | Unset):
        new (bool | Unset):
        num_recent (int | Unset):
        page (int | Unset):
        page_size (int | Unset):
        query (str | Unset):
        random (bool | Unset):
        rating (int | Unset):
        rating_gte (int | Unset):
        rating_lte (int | Unset):
        sort_order (str | Unset):
        timescooked (int | Unset):
        timescooked_gte (int | Unset):
        timescooked_lte (int | Unset):
        units (int | Unset):
        updatedon (datetime.date | Unset):
        updatedon_gte (datetime.date | Unset):
        updatedon_lte (datetime.date | Unset):
        viewedon_gte (datetime.date | Unset):
        viewedon_lte (datetime.date | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedRecipeOverviewList]
    """

    kwargs = _get_kwargs(
        books=books,
        books_and=books_and,
        books_and_not=books_and_not,
        books_or=books_or,
        books_or_not=books_or_not,
        cookedon_gte=cookedon_gte,
        cookedon_lte=cookedon_lte,
        createdby=createdby,
        createdon=createdon,
        createdon_gte=createdon_gte,
        createdon_lte=createdon_lte,
        filter_=filter_,
        foods=foods,
        foods_and=foods_and,
        foods_and_not=foods_and_not,
        foods_or=foods_or,
        foods_or_not=foods_or_not,
        internal=internal,
        keywords=keywords,
        keywords_and=keywords_and,
        keywords_and_not=keywords_and_not,
        keywords_or=keywords_or,
        keywords_or_not=keywords_or_not,
        makenow=makenow,
        new=new,
        num_recent=num_recent,
        page=page,
        page_size=page_size,
        query=query,
        random=random,
        rating=rating,
        rating_gte=rating_gte,
        rating_lte=rating_lte,
        sort_order=sort_order,
        timescooked=timescooked,
        timescooked_gte=timescooked_gte,
        timescooked_lte=timescooked_lte,
        units=units,
        updatedon=updatedon,
        updatedon_gte=updatedon_gte,
        updatedon_lte=updatedon_lte,
        viewedon_gte=viewedon_gte,
        viewedon_lte=viewedon_lte,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    books: list[int] | Unset = UNSET,
    books_and: list[int] | Unset = UNSET,
    books_and_not: list[int] | Unset = UNSET,
    books_or: list[int] | Unset = UNSET,
    books_or_not: list[int] | Unset = UNSET,
    cookedon_gte: datetime.date | Unset = UNSET,
    cookedon_lte: datetime.date | Unset = UNSET,
    createdby: int | Unset = UNSET,
    createdon: datetime.date | Unset = UNSET,
    createdon_gte: datetime.date | Unset = UNSET,
    createdon_lte: datetime.date | Unset = UNSET,
    filter_: int | Unset = UNSET,
    foods: list[int] | Unset = UNSET,
    foods_and: list[int] | Unset = UNSET,
    foods_and_not: list[int] | Unset = UNSET,
    foods_or: list[int] | Unset = UNSET,
    foods_or_not: list[int] | Unset = UNSET,
    internal: bool | Unset = UNSET,
    keywords: list[int] | Unset = UNSET,
    keywords_and: list[int] | Unset = UNSET,
    keywords_and_not: list[int] | Unset = UNSET,
    keywords_or: list[int] | Unset = UNSET,
    keywords_or_not: list[int] | Unset = UNSET,
    makenow: bool | Unset = UNSET,
    new: bool | Unset = UNSET,
    num_recent: int | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    query: str | Unset = UNSET,
    random: bool | Unset = UNSET,
    rating: int | Unset = UNSET,
    rating_gte: int | Unset = UNSET,
    rating_lte: int | Unset = UNSET,
    sort_order: str | Unset = UNSET,
    timescooked: int | Unset = UNSET,
    timescooked_gte: int | Unset = UNSET,
    timescooked_lte: int | Unset = UNSET,
    units: int | Unset = UNSET,
    updatedon: datetime.date | Unset = UNSET,
    updatedon_gte: datetime.date | Unset = UNSET,
    updatedon_lte: datetime.date | Unset = UNSET,
    viewedon_gte: datetime.date | Unset = UNSET,
    viewedon_lte: datetime.date | Unset = UNSET,
) -> PaginatedRecipeOverviewList | None:
    """logs request counts to redis cache total/per user/

    Args:
        books (list[int] | Unset):
        books_and (list[int] | Unset):
        books_and_not (list[int] | Unset):
        books_or (list[int] | Unset):
        books_or_not (list[int] | Unset):
        cookedon_gte (datetime.date | Unset):
        cookedon_lte (datetime.date | Unset):
        createdby (int | Unset):
        createdon (datetime.date | Unset):
        createdon_gte (datetime.date | Unset):
        createdon_lte (datetime.date | Unset):
        filter_ (int | Unset):
        foods (list[int] | Unset):
        foods_and (list[int] | Unset):
        foods_and_not (list[int] | Unset):
        foods_or (list[int] | Unset):
        foods_or_not (list[int] | Unset):
        internal (bool | Unset):
        keywords (list[int] | Unset):
        keywords_and (list[int] | Unset):
        keywords_and_not (list[int] | Unset):
        keywords_or (list[int] | Unset):
        keywords_or_not (list[int] | Unset):
        makenow (bool | Unset):
        new (bool | Unset):
        num_recent (int | Unset):
        page (int | Unset):
        page_size (int | Unset):
        query (str | Unset):
        random (bool | Unset):
        rating (int | Unset):
        rating_gte (int | Unset):
        rating_lte (int | Unset):
        sort_order (str | Unset):
        timescooked (int | Unset):
        timescooked_gte (int | Unset):
        timescooked_lte (int | Unset):
        units (int | Unset):
        updatedon (datetime.date | Unset):
        updatedon_gte (datetime.date | Unset):
        updatedon_lte (datetime.date | Unset):
        viewedon_gte (datetime.date | Unset):
        viewedon_lte (datetime.date | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedRecipeOverviewList
    """

    return sync_detailed(
        client=client,
        books=books,
        books_and=books_and,
        books_and_not=books_and_not,
        books_or=books_or,
        books_or_not=books_or_not,
        cookedon_gte=cookedon_gte,
        cookedon_lte=cookedon_lte,
        createdby=createdby,
        createdon=createdon,
        createdon_gte=createdon_gte,
        createdon_lte=createdon_lte,
        filter_=filter_,
        foods=foods,
        foods_and=foods_and,
        foods_and_not=foods_and_not,
        foods_or=foods_or,
        foods_or_not=foods_or_not,
        internal=internal,
        keywords=keywords,
        keywords_and=keywords_and,
        keywords_and_not=keywords_and_not,
        keywords_or=keywords_or,
        keywords_or_not=keywords_or_not,
        makenow=makenow,
        new=new,
        num_recent=num_recent,
        page=page,
        page_size=page_size,
        query=query,
        random=random,
        rating=rating,
        rating_gte=rating_gte,
        rating_lte=rating_lte,
        sort_order=sort_order,
        timescooked=timescooked,
        timescooked_gte=timescooked_gte,
        timescooked_lte=timescooked_lte,
        units=units,
        updatedon=updatedon,
        updatedon_gte=updatedon_gte,
        updatedon_lte=updatedon_lte,
        viewedon_gte=viewedon_gte,
        viewedon_lte=viewedon_lte,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    books: list[int] | Unset = UNSET,
    books_and: list[int] | Unset = UNSET,
    books_and_not: list[int] | Unset = UNSET,
    books_or: list[int] | Unset = UNSET,
    books_or_not: list[int] | Unset = UNSET,
    cookedon_gte: datetime.date | Unset = UNSET,
    cookedon_lte: datetime.date | Unset = UNSET,
    createdby: int | Unset = UNSET,
    createdon: datetime.date | Unset = UNSET,
    createdon_gte: datetime.date | Unset = UNSET,
    createdon_lte: datetime.date | Unset = UNSET,
    filter_: int | Unset = UNSET,
    foods: list[int] | Unset = UNSET,
    foods_and: list[int] | Unset = UNSET,
    foods_and_not: list[int] | Unset = UNSET,
    foods_or: list[int] | Unset = UNSET,
    foods_or_not: list[int] | Unset = UNSET,
    internal: bool | Unset = UNSET,
    keywords: list[int] | Unset = UNSET,
    keywords_and: list[int] | Unset = UNSET,
    keywords_and_not: list[int] | Unset = UNSET,
    keywords_or: list[int] | Unset = UNSET,
    keywords_or_not: list[int] | Unset = UNSET,
    makenow: bool | Unset = UNSET,
    new: bool | Unset = UNSET,
    num_recent: int | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    query: str | Unset = UNSET,
    random: bool | Unset = UNSET,
    rating: int | Unset = UNSET,
    rating_gte: int | Unset = UNSET,
    rating_lte: int | Unset = UNSET,
    sort_order: str | Unset = UNSET,
    timescooked: int | Unset = UNSET,
    timescooked_gte: int | Unset = UNSET,
    timescooked_lte: int | Unset = UNSET,
    units: int | Unset = UNSET,
    updatedon: datetime.date | Unset = UNSET,
    updatedon_gte: datetime.date | Unset = UNSET,
    updatedon_lte: datetime.date | Unset = UNSET,
    viewedon_gte: datetime.date | Unset = UNSET,
    viewedon_lte: datetime.date | Unset = UNSET,
) -> Response[PaginatedRecipeOverviewList]:
    """logs request counts to redis cache total/per user/

    Args:
        books (list[int] | Unset):
        books_and (list[int] | Unset):
        books_and_not (list[int] | Unset):
        books_or (list[int] | Unset):
        books_or_not (list[int] | Unset):
        cookedon_gte (datetime.date | Unset):
        cookedon_lte (datetime.date | Unset):
        createdby (int | Unset):
        createdon (datetime.date | Unset):
        createdon_gte (datetime.date | Unset):
        createdon_lte (datetime.date | Unset):
        filter_ (int | Unset):
        foods (list[int] | Unset):
        foods_and (list[int] | Unset):
        foods_and_not (list[int] | Unset):
        foods_or (list[int] | Unset):
        foods_or_not (list[int] | Unset):
        internal (bool | Unset):
        keywords (list[int] | Unset):
        keywords_and (list[int] | Unset):
        keywords_and_not (list[int] | Unset):
        keywords_or (list[int] | Unset):
        keywords_or_not (list[int] | Unset):
        makenow (bool | Unset):
        new (bool | Unset):
        num_recent (int | Unset):
        page (int | Unset):
        page_size (int | Unset):
        query (str | Unset):
        random (bool | Unset):
        rating (int | Unset):
        rating_gte (int | Unset):
        rating_lte (int | Unset):
        sort_order (str | Unset):
        timescooked (int | Unset):
        timescooked_gte (int | Unset):
        timescooked_lte (int | Unset):
        units (int | Unset):
        updatedon (datetime.date | Unset):
        updatedon_gte (datetime.date | Unset):
        updatedon_lte (datetime.date | Unset):
        viewedon_gte (datetime.date | Unset):
        viewedon_lte (datetime.date | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedRecipeOverviewList]
    """

    kwargs = _get_kwargs(
        books=books,
        books_and=books_and,
        books_and_not=books_and_not,
        books_or=books_or,
        books_or_not=books_or_not,
        cookedon_gte=cookedon_gte,
        cookedon_lte=cookedon_lte,
        createdby=createdby,
        createdon=createdon,
        createdon_gte=createdon_gte,
        createdon_lte=createdon_lte,
        filter_=filter_,
        foods=foods,
        foods_and=foods_and,
        foods_and_not=foods_and_not,
        foods_or=foods_or,
        foods_or_not=foods_or_not,
        internal=internal,
        keywords=keywords,
        keywords_and=keywords_and,
        keywords_and_not=keywords_and_not,
        keywords_or=keywords_or,
        keywords_or_not=keywords_or_not,
        makenow=makenow,
        new=new,
        num_recent=num_recent,
        page=page,
        page_size=page_size,
        query=query,
        random=random,
        rating=rating,
        rating_gte=rating_gte,
        rating_lte=rating_lte,
        sort_order=sort_order,
        timescooked=timescooked,
        timescooked_gte=timescooked_gte,
        timescooked_lte=timescooked_lte,
        units=units,
        updatedon=updatedon,
        updatedon_gte=updatedon_gte,
        updatedon_lte=updatedon_lte,
        viewedon_gte=viewedon_gte,
        viewedon_lte=viewedon_lte,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    books: list[int] | Unset = UNSET,
    books_and: list[int] | Unset = UNSET,
    books_and_not: list[int] | Unset = UNSET,
    books_or: list[int] | Unset = UNSET,
    books_or_not: list[int] | Unset = UNSET,
    cookedon_gte: datetime.date | Unset = UNSET,
    cookedon_lte: datetime.date | Unset = UNSET,
    createdby: int | Unset = UNSET,
    createdon: datetime.date | Unset = UNSET,
    createdon_gte: datetime.date | Unset = UNSET,
    createdon_lte: datetime.date | Unset = UNSET,
    filter_: int | Unset = UNSET,
    foods: list[int] | Unset = UNSET,
    foods_and: list[int] | Unset = UNSET,
    foods_and_not: list[int] | Unset = UNSET,
    foods_or: list[int] | Unset = UNSET,
    foods_or_not: list[int] | Unset = UNSET,
    internal: bool | Unset = UNSET,
    keywords: list[int] | Unset = UNSET,
    keywords_and: list[int] | Unset = UNSET,
    keywords_and_not: list[int] | Unset = UNSET,
    keywords_or: list[int] | Unset = UNSET,
    keywords_or_not: list[int] | Unset = UNSET,
    makenow: bool | Unset = UNSET,
    new: bool | Unset = UNSET,
    num_recent: int | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    query: str | Unset = UNSET,
    random: bool | Unset = UNSET,
    rating: int | Unset = UNSET,
    rating_gte: int | Unset = UNSET,
    rating_lte: int | Unset = UNSET,
    sort_order: str | Unset = UNSET,
    timescooked: int | Unset = UNSET,
    timescooked_gte: int | Unset = UNSET,
    timescooked_lte: int | Unset = UNSET,
    units: int | Unset = UNSET,
    updatedon: datetime.date | Unset = UNSET,
    updatedon_gte: datetime.date | Unset = UNSET,
    updatedon_lte: datetime.date | Unset = UNSET,
    viewedon_gte: datetime.date | Unset = UNSET,
    viewedon_lte: datetime.date | Unset = UNSET,
) -> PaginatedRecipeOverviewList | None:
    """logs request counts to redis cache total/per user/

    Args:
        books (list[int] | Unset):
        books_and (list[int] | Unset):
        books_and_not (list[int] | Unset):
        books_or (list[int] | Unset):
        books_or_not (list[int] | Unset):
        cookedon_gte (datetime.date | Unset):
        cookedon_lte (datetime.date | Unset):
        createdby (int | Unset):
        createdon (datetime.date | Unset):
        createdon_gte (datetime.date | Unset):
        createdon_lte (datetime.date | Unset):
        filter_ (int | Unset):
        foods (list[int] | Unset):
        foods_and (list[int] | Unset):
        foods_and_not (list[int] | Unset):
        foods_or (list[int] | Unset):
        foods_or_not (list[int] | Unset):
        internal (bool | Unset):
        keywords (list[int] | Unset):
        keywords_and (list[int] | Unset):
        keywords_and_not (list[int] | Unset):
        keywords_or (list[int] | Unset):
        keywords_or_not (list[int] | Unset):
        makenow (bool | Unset):
        new (bool | Unset):
        num_recent (int | Unset):
        page (int | Unset):
        page_size (int | Unset):
        query (str | Unset):
        random (bool | Unset):
        rating (int | Unset):
        rating_gte (int | Unset):
        rating_lte (int | Unset):
        sort_order (str | Unset):
        timescooked (int | Unset):
        timescooked_gte (int | Unset):
        timescooked_lte (int | Unset):
        units (int | Unset):
        updatedon (datetime.date | Unset):
        updatedon_gte (datetime.date | Unset):
        updatedon_lte (datetime.date | Unset):
        viewedon_gte (datetime.date | Unset):
        viewedon_lte (datetime.date | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedRecipeOverviewList
    """

    return (
        await asyncio_detailed(
            client=client,
            books=books,
            books_and=books_and,
            books_and_not=books_and_not,
            books_or=books_or,
            books_or_not=books_or_not,
            cookedon_gte=cookedon_gte,
            cookedon_lte=cookedon_lte,
            createdby=createdby,
            createdon=createdon,
            createdon_gte=createdon_gte,
            createdon_lte=createdon_lte,
            filter_=filter_,
            foods=foods,
            foods_and=foods_and,
            foods_and_not=foods_and_not,
            foods_or=foods_or,
            foods_or_not=foods_or_not,
            internal=internal,
            keywords=keywords,
            keywords_and=keywords_and,
            keywords_and_not=keywords_and_not,
            keywords_or=keywords_or,
            keywords_or_not=keywords_or_not,
            makenow=makenow,
            new=new,
            num_recent=num_recent,
            page=page,
            page_size=page_size,
            query=query,
            random=random,
            rating=rating,
            rating_gte=rating_gte,
            rating_lte=rating_lte,
            sort_order=sort_order,
            timescooked=timescooked,
            timescooked_gte=timescooked_gte,
            timescooked_lte=timescooked_lte,
            units=units,
            updatedon=updatedon,
            updatedon_gte=updatedon_gte,
            updatedon_lte=updatedon_lte,
            viewedon_gte=viewedon_gte,
            viewedon_lte=viewedon_lte,
        )
    ).parsed



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExportLog")


@_attrs_define
class ExportLog:
    """
    Attributes:
        type_ (str):
        created_by (int):
        created_at (datetime.datetime):
        id (int | Unset):
        msg (str | Unset):
        running (bool | Unset):
        total_recipes (int | Unset):
        exported_recipes (int | Unset):
        cache_duration (int | Unset):
        possibly_not_expired (bool | Unset):
    """

    type_: str
    created_by: int
    created_at: datetime.datetime
    id: int | Unset = UNSET
    msg: str | Unset = UNSET
    running: bool | Unset = UNSET
    total_recipes: int | Unset = UNSET
    exported_recipes: int | Unset = UNSET
    cache_duration: int | Unset = UNSET
    possibly_not_expired: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        created_by = self.created_by

        created_at = self.created_at.isoformat()

        id = self.id

        msg = self.msg

        running = self.running

        total_recipes = self.total_recipes

        exported_recipes = self.exported_recipes

        cache_duration = self.cache_duration

        possibly_not_expired = self.possibly_not_expired

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
                "created_by": created_by,
                "created_at": created_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if msg is not UNSET:
            field_dict["msg"] = msg
        if running is not UNSET:
            field_dict["running"] = running
        if total_recipes is not UNSET:
            field_dict["total_recipes"] = total_recipes
        if exported_recipes is not UNSET:
            field_dict["exported_recipes"] = exported_recipes
        if cache_duration is not UNSET:
            field_dict["cache_duration"] = cache_duration
        if possibly_not_expired is not UNSET:
            field_dict["possibly_not_expired"] = possibly_not_expired

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("type", (None, str(self.type_).encode(), "text/plain")))

        files.append(("created_by", (None, str(self.created_by).encode(), "text/plain")))

        files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.msg, Unset):
            files.append(("msg", (None, str(self.msg).encode(), "text/plain")))

        if not isinstance(self.running, Unset):
            files.append(("running", (None, str(self.running).encode(), "text/plain")))

        if not isinstance(self.total_recipes, Unset):
            files.append(("total_recipes", (None, str(self.total_recipes).encode(), "text/plain")))

        if not isinstance(self.exported_recipes, Unset):
            files.append(("exported_recipes", (None, str(self.exported_recipes).encode(), "text/plain")))

        if not isinstance(self.cache_duration, Unset):
            files.append(("cache_duration", (None, str(self.cache_duration).encode(), "text/plain")))

        if not isinstance(self.possibly_not_expired, Unset):
            files.append(("possibly_not_expired", (None, str(self.possibly_not_expired).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = d.pop("type")

        created_by = d.pop("created_by")

        created_at = isoparse(d.pop("created_at"))

        id = d.pop("id", UNSET)

        msg = d.pop("msg", UNSET)

        running = d.pop("running", UNSET)

        total_recipes = d.pop("total_recipes", UNSET)

        exported_recipes = d.pop("exported_recipes", UNSET)

        cache_duration = d.pop("cache_duration", UNSET)

        possibly_not_expired = d.pop("possibly_not_expired", UNSET)

        export_log = cls(
            type_=type_,
            created_by=created_by,
            created_at=created_at,
            id=id,
            msg=msg,
            running=running,
            total_recipes=total_recipes,
            exported_recipes=exported_recipes,
            cache_duration=cache_duration,
            possibly_not_expired=possibly_not_expired,
        )

        export_log.additional_properties = d
        return export_log

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any
from urllib.parse import quote

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.food import Food
from ...types import UNSET, Response


def _get_kwargs(
    id: int,
    *,
    body: Food | Food | Food | Unset = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "put",
        "url": "/api/food/{id}/".format(
            id=quote(str(id), safe=""),
        ),
    }

    if isinstance(body, Food):
        _kwargs["json"] = body.to_dict()

        headers["Content-Type"] = "application/json"
    if isinstance(body, Food):
        _kwargs["data"] = body.to_dict()

        headers["Content-Type"] = "application/x-www-form-urlencoded"
    if isinstance(body, Food):
        _kwargs["files"] = body.to_multipart()

        headers["Content-Type"] = "multipart/form-data"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Food | None:
    if response.status_code == 200:
        response_200 = Food.from_dict(response.json())

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[Food]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: int,
    *,
    client: AuthenticatedClient,
    body: Food | Food | Food | Unset = UNSET,
) -> Response[Food]:
    """logs request counts to redis cache total/per user/

    Args:
        id (int):
        body (Food): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        body (Food): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        body (Food): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Food]
    """

    kwargs = _get_kwargs(
        id=id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: int,
    *,
    client: AuthenticatedClient,
    body: Food | Food | Food | Unset = UNSET,
) -> Food | None:
    """logs request counts to redis cache total/per user/

    Args:
        id (int):
        body (Food): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        body (Food): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        body (Food): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Food
    """

    return sync_detailed(
        id=id,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    id: int,
    *,
    client: AuthenticatedClient,
    body: Food | Food | Food | Unset = UNSET,
) -> Response[Food]:
    """logs request counts to redis cache total/per user/

    Args:
        id (int):
        body (Food): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        body (Food): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        body (Food): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Food]
    """

    kwargs = _get_kwargs(
        id=id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: int,
    *,
    client: AuthenticatedClient,
    body: Food | Food | Food | Unset = UNSET,
) -> Food | None:
    """logs request counts to redis cache total/per user/

    Args:
        id (int):
        body (Food): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        body (Food): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        body (Food): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Food
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            body=body,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="RecipeShoppingUpdate")


@_attrs_define
class RecipeShoppingUpdate:
    """
    Attributes:
        ingredients (list[int | None]):
        id (int | Unset):
        list_recipe (int | None | Unset): Existing shopping list to update
        servings (int | None | Unset): Providing a list_recipe ID and servings of 0 will delete that shopping list.
            Default: 1.
    """

    ingredients: list[int | None]
    id: int | Unset = UNSET
    list_recipe: int | None | Unset = UNSET
    servings: int | None | Unset = 1
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        ingredients = []
        for ingredients_item_data in self.ingredients:
            ingredients_item: int | None
            ingredients_item = ingredients_item_data
            ingredients.append(ingredients_item)

        id = self.id

        list_recipe: int | None | Unset
        if isinstance(self.list_recipe, Unset):
            list_recipe = UNSET
        else:
            list_recipe = self.list_recipe

        servings: int | None | Unset
        if isinstance(self.servings, Unset):
            servings = UNSET
        else:
            servings = self.servings

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "ingredients": ingredients,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if list_recipe is not UNSET:
            field_dict["list_recipe"] = list_recipe
        if servings is not UNSET:
            field_dict["servings"] = servings

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        for ingredients_item_element in self.ingredients:
            if isinstance(ingredients_item_element, int):
                files.append(("ingredients", (None, str(ingredients_item_element).encode(), "text/plain")))
            else:
                files.append(("ingredients", (None, str(ingredients_item_element).encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.list_recipe, Unset):
            if isinstance(self.list_recipe, int):
                files.append(("list_recipe", (None, str(self.list_recipe).encode(), "text/plain")))
            else:
                files.append(("list_recipe", (None, str(self.list_recipe).encode(), "text/plain")))

        if not isinstance(self.servings, Unset):
            if isinstance(self.servings, int):
                files.append(("servings", (None, str(self.servings).encode(), "text/plain")))
            else:
                files.append(("servings", (None, str(self.servings).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        ingredients = []
        _ingredients = d.pop("ingredients")
        for ingredients_item_data in _ingredients:

            def _parse_ingredients_item(data: object) -> int | None:
                if data is None:
                    return data
                return cast(int | None, data)

            ingredients_item = _parse_ingredients_item(ingredients_item_data)

            ingredients.append(ingredients_item)

        id = d.pop("id", UNSET)

        def _parse_list_recipe(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        list_recipe = _parse_list_recipe(d.pop("list_recipe", UNSET))

        def _parse_servings(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        servings = _parse_servings(d.pop("servings", UNSET))

        recipe_shopping_update = cls(
            ingredients=ingredients,
            id=id,
            list_recipe=list_recipe,
            servings=servings,
        )

        recipe_shopping_update.additional_properties = d
        return recipe_shopping_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types

T = TypeVar("T", bound="ShoppingListEntryBulk")


@_attrs_define
class ShoppingListEntryBulk:
    """
    Attributes:
        ids (list[Any]):
        checked (bool):
        timestamp (datetime.datetime):
    """

    ids: list[Any]
    checked: bool
    timestamp: datetime.datetime
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        ids = self.ids

        checked = self.checked

        timestamp = self.timestamp.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "ids": ids,
                "checked": checked,
                "timestamp": timestamp,
            }
        )

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        for ids_item_element in self.ids:
            files.append(("ids", (None, str(ids_item_element).encode(), "text/plain")))

        files.append(("checked", (None, str(self.checked).encode(), "text/plain")))

        files.append(("timestamp", (None, self.timestamp.isoformat().encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        ids = cast(list[Any], d.pop("ids"))

        checked = d.pop("checked")

        timestamp = isoparse(d.pop("timestamp"))

        shopping_list_entry_bulk = cls(
            ids=ids,
            checked=checked,
            timestamp=timestamp,
        )

        shopping_list_entry_bulk.additional_properties = d
        return shopping_list_entry_bulk

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SearchEnum(str, Enum):
    PHRASE = "phrase"
    PLAIN = "plain"
    RAW = "raw"
    WEBSEARCH = "websearch"

    def __str__(self) -> str:
        return str(self.value)



"""Contains all the data models used in inputs/outputs"""

from .access_token import AccessToken
from .ai_import import AiImport
from .ai_log import AiLog
from .ai_provider import AiProvider
from .api_automation_list_type_item import ApiAutomationListTypeItem
from .api_custom_filter_list_type_item import ApiCustomFilterListTypeItem
from .api_property_type_list_category_item import ApiPropertyTypeListCategoryItem
from .api_recipe_book_list_order_direction import ApiRecipeBookListOrderDirection
from .api_recipe_book_list_order_field import ApiRecipeBookListOrderField
from .auth_token import AuthToken
from .auto_meal_plan import AutoMealPlan
from .automation import Automation
from .automation_type_enum import AutomationTypeEnum
from .bookmarklet_import import BookmarkletImport
from .bookmarklet_import_list import BookmarkletImportList
from .connector_config import ConnectorConfig
from .connector_config_type_enum import ConnectorConfigTypeEnum
from .cook_log import CookLog
from .custom_filter import CustomFilter
from .default_page_enum import DefaultPageEnum
from .delete_enum import DeleteEnum
from .export_log import ExportLog
from .export_request import ExportRequest
from .fdc_query import FdcQuery
from .fdc_query_foods import FdcQueryFoods
from .food import Food
from .food_batch_update import FoodBatchUpdate
from .food_inherit_field import FoodInheritField
from .food_shopping_update import FoodShoppingUpdate
from .food_simple import FoodSimple
from .generic_model_reference import GenericModelReference
from .group import Group
from .import_log import ImportLog
from .import_open_data import ImportOpenData
from .import_open_data_meta_data import ImportOpenDataMetaData
from .import_open_data_response import ImportOpenDataResponse
from .import_open_data_response_detail import ImportOpenDataResponseDetail
from .import_open_data_version_meta_data import ImportOpenDataVersionMetaData
from .ingredient import Ingredient
from .ingredient_string import IngredientString
from .invite_link import InviteLink
from .keyword import Keyword
from .keyword_label import KeywordLabel
from .localization import Localization
from .meal_plan import MealPlan
from .meal_type import MealType
from .method_enum import MethodEnum
from .nutrition_information import NutritionInformation
from .paginated_ai_log_list import PaginatedAiLogList
from .paginated_ai_provider_list import PaginatedAiProviderList
from .paginated_automation_list import PaginatedAutomationList
from .paginated_bookmarklet_import_list_list import PaginatedBookmarkletImportListList
from .paginated_connector_config_list import PaginatedConnectorConfigList
from .paginated_cook_log_list import PaginatedCookLogList
from .paginated_custom_filter_list import PaginatedCustomFilterList
from .paginated_export_log_list import PaginatedExportLogList
from .paginated_food_list import PaginatedFoodList
from .paginated_generic_model_reference_list import PaginatedGenericModelReferenceList
from .paginated_import_log_list import PaginatedImportLogList
from .paginated_ingredient_list import PaginatedIngredientList
from .paginated_invite_link_list import PaginatedInviteLinkList
from .paginated_keyword_list import PaginatedKeywordList
from .paginated_meal_plan_list import PaginatedMealPlanList
from .paginated_meal_type_list import PaginatedMealTypeList
from .paginated_property_list import PaginatedPropertyList
from .paginated_property_type_list import PaginatedPropertyTypeList
from .paginated_recipe_book_entry_list import PaginatedRecipeBookEntryList
from .paginated_recipe_book_list import PaginatedRecipeBookList
from .paginated_recipe_import_list import PaginatedRecipeImportList
from .paginated_recipe_overview_list import PaginatedRecipeOverviewList
from .paginated_shopping_list_entry_list import PaginatedShoppingListEntryList
from .paginated_shopping_list_recipe_list import PaginatedShoppingListRecipeList
from .paginated_space_list import PaginatedSpaceList
from .paginated_step_list import PaginatedStepList
from .paginated_storage_list import PaginatedStorageList
from .paginated_supermarket_category_list import PaginatedSupermarketCategoryList
from .paginated_supermarket_category_relation_list import PaginatedSupermarketCategoryRelationList
from .paginated_supermarket_list import PaginatedSupermarketList
from .paginated_sync_list import PaginatedSyncList
from .paginated_sync_log_list import PaginatedSyncLogList
from .paginated_unit_conversion_list import PaginatedUnitConversionList
from .paginated_unit_list import PaginatedUnitList
from .paginated_user_file_list import PaginatedUserFileList
from .paginated_user_space_list import PaginatedUserSpaceList
from .paginated_view_log_list import PaginatedViewLogList
from .parsed_ingredient import ParsedIngredient
from .patched_access_token import PatchedAccessToken
from .patched_ai_provider import PatchedAiProvider
from .patched_automation import PatchedAutomation
from .patched_bookmarklet_import import PatchedBookmarkletImport
from .patched_connector_config import PatchedConnectorConfig
from .patched_cook_log import PatchedCookLog
from .patched_custom_filter import PatchedCustomFilter
from .patched_export_log import PatchedExportLog
from .patched_food import PatchedFood
from .patched_import_log import PatchedImportLog
from .patched_ingredient import PatchedIngredient
from .patched_invite_link import PatchedInviteLink
from .patched_keyword import PatchedKeyword
from .patched_meal_plan import PatchedMealPlan
from .patched_meal_type import PatchedMealType
from .patched_property import PatchedProperty
from .patched_property_type import PatchedPropertyType
from .patched_recipe import PatchedRecipe
from .patched_recipe_book import PatchedRecipeBook
from .patched_recipe_book_entry import PatchedRecipeBookEntry
from .patched_recipe_import import PatchedRecipeImport
from .patched_search_preference import PatchedSearchPreference
from .patched_shopping_list_entry import PatchedShoppingListEntry
from .patched_shopping_list_recipe import PatchedShoppingListRecipe
from .patched_space import PatchedSpace
from .patched_step import PatchedStep
from .patched_storage import PatchedStorage
from .patched_supermarket import PatchedSupermarket
from .patched_supermarket_category import PatchedSupermarketCategory
from .patched_supermarket_category_relation import PatchedSupermarketCategoryRelation
from .patched_sync import PatchedSync
from .patched_unit import PatchedUnit
from .patched_unit_conversion import PatchedUnitConversion
from .patched_user import PatchedUser
from .patched_user_file import PatchedUserFile
from .patched_user_preference import PatchedUserPreference
from .patched_user_space import PatchedUserSpace
from .patched_view_log import PatchedViewLog
from .property_ import Property
from .property_type import PropertyType
from .recipe import Recipe
from .recipe_batch_update import RecipeBatchUpdate
from .recipe_book import RecipeBook
from .recipe_book_entry import RecipeBookEntry
from .recipe_flat import RecipeFlat
from .recipe_from_source import RecipeFromSource
from .recipe_from_source_response import RecipeFromSourceResponse
from .recipe_image import RecipeImage
from .recipe_import import RecipeImport
from .recipe_overview import RecipeOverview
from .recipe_shopping_update import RecipeShoppingUpdate
from .recipe_simple import RecipeSimple
from .search_enum import SearchEnum
from .search_fields import SearchFields
from .search_preference import SearchPreference
from .server_settings import ServerSettings
from .share_link import ShareLink
from .shopping_list_entry import ShoppingListEntry
from .shopping_list_entry_bulk import ShoppingListEntryBulk
from .shopping_list_entry_bulk_create import ShoppingListEntryBulkCreate
from .shopping_list_entry_simple_create import ShoppingListEntrySimpleCreate
from .shopping_list_recipe import ShoppingListRecipe
from .source_import_duplicate import SourceImportDuplicate
from .source_import_food import SourceImportFood
from .source_import_ingredient import SourceImportIngredient
from .source_import_keyword import SourceImportKeyword
from .source_import_property import SourceImportProperty
from .source_import_property_type import SourceImportPropertyType
from .source_import_recipe import SourceImportRecipe
from .source_import_step import SourceImportStep
from .source_import_unit import SourceImportUnit
from .space import Space
from .space_nav_text_color_enum import SpaceNavTextColorEnum
from .space_theme_enum import SpaceThemeEnum
from .step import Step
from .storage import Storage
from .supermarket import Supermarket
from .supermarket_category import SupermarketCategory
from .supermarket_category_relation import SupermarketCategoryRelation
from .sync import Sync
from .sync_log import SyncLog
from .theme_enum import ThemeEnum
from .unit import Unit
from .unit_conversion import UnitConversion
from .user import User
from .user_file import UserFile
from .user_file_view import UserFileView
from .user_preference import UserPreference
from .user_preference_nav_text_color_enum import UserPreferenceNavTextColorEnum
from .user_space import UserSpace
from .view_log import ViewLog

__all__ = (
    "AccessToken",
    "AiImport",
    "AiLog",
    "AiProvider",
    "ApiAutomationListTypeItem",
    "ApiCustomFilterListTypeItem",
    "ApiPropertyTypeListCategoryItem",
    "ApiRecipeBookListOrderDirection",
    "ApiRecipeBookListOrderField",
    "AuthToken",
    "Automation",
    "AutomationTypeEnum",
    "AutoMealPlan",
    "BookmarkletImport",
    "BookmarkletImportList",
    "ConnectorConfig",
    "ConnectorConfigTypeEnum",
    "CookLog",
    "CustomFilter",
    "DefaultPageEnum",
    "DeleteEnum",
    "ExportLog",
    "ExportRequest",
    "FdcQuery",
    "FdcQueryFoods",
    "Food",
    "FoodBatchUpdate",
    "FoodInheritField",
    "FoodShoppingUpdate",
    "FoodSimple",
    "GenericModelReference",
    "Group",
    "ImportLog",
    "ImportOpenData",
    "ImportOpenDataMetaData",
    "ImportOpenDataResponse",
    "ImportOpenDataResponseDetail",
    "ImportOpenDataVersionMetaData",
    "Ingredient",
    "IngredientString",
    "InviteLink",
    "Keyword",
    "KeywordLabel",
    "Localization",
    "MealPlan",
    "MealType",
    "MethodEnum",
    "NutritionInformation",
    "PaginatedAiLogList",
    "PaginatedAiProviderList",
    "PaginatedAutomationList",
    "PaginatedBookmarkletImportListList",
    "PaginatedConnectorConfigList",
    "PaginatedCookLogList",
    "PaginatedCustomFilterList",
    "PaginatedExportLogList",
    "PaginatedFoodList",
    "PaginatedGenericModelReferenceList",
    "PaginatedImportLogList",
    "PaginatedIngredientList",
    "PaginatedInviteLinkList",
    "PaginatedKeywordList",
    "PaginatedMealPlanList",
    "PaginatedMealTypeList",
    "PaginatedPropertyList",
    "PaginatedPropertyTypeList",
    "PaginatedRecipeBookEntryList",
    "PaginatedRecipeBookList",
    "PaginatedRecipeImportList",
    "PaginatedRecipeOverviewList",
    "PaginatedShoppingListEntryList",
    "PaginatedShoppingListRecipeList",
    "PaginatedSpaceList",
    "PaginatedStepList",
    "PaginatedStorageList",
    "PaginatedSupermarketCategoryList",
    "PaginatedSupermarketCategoryRelationList",
    "PaginatedSupermarketList",
    "PaginatedSyncList",
    "PaginatedSyncLogList",
    "PaginatedUnitConversionList",
    "PaginatedUnitList",
    "PaginatedUserFileList",
    "PaginatedUserSpaceList",
    "PaginatedViewLogList",
    "ParsedIngredient",
    "PatchedAccessToken",
    "PatchedAiProvider",
    "PatchedAutomation",
    "PatchedBookmarkletImport",
    "PatchedConnectorConfig",
    "PatchedCookLog",
    "PatchedCustomFilter",
    "PatchedExportLog",
    "PatchedFood",
    "PatchedImportLog",
    "PatchedIngredient",
    "PatchedInviteLink",
    "PatchedKeyword",
    "PatchedMealPlan",
    "PatchedMealType",
    "PatchedProperty",
    "PatchedPropertyType",
    "PatchedRecipe",
    "PatchedRecipeBook",
    "PatchedRecipeBookEntry",
    "PatchedRecipeImport",
    "PatchedSearchPreference",
    "PatchedShoppingListEntry",
    "PatchedShoppingListRecipe",
    "PatchedSpace",
    "PatchedStep",
    "PatchedStorage",
    "PatchedSupermarket",
    "PatchedSupermarketCategory",
    "PatchedSupermarketCategoryRelation",
    "PatchedSync",
    "PatchedUnit",
    "PatchedUnitConversion",
    "PatchedUser",
    "PatchedUserFile",
    "PatchedUserPreference",
    "PatchedUserSpace",
    "PatchedViewLog",
    "Property",
    "PropertyType",
    "Recipe",
    "RecipeBatchUpdate",
    "RecipeBook",
    "RecipeBookEntry",
    "RecipeFlat",
    "RecipeFromSource",
    "RecipeFromSourceResponse",
    "RecipeImage",
    "RecipeImport",
    "RecipeOverview",
    "RecipeShoppingUpdate",
    "RecipeSimple",
    "SearchEnum",
    "SearchFields",
    "SearchPreference",
    "ServerSettings",
    "ShareLink",
    "ShoppingListEntry",
    "ShoppingListEntryBulk",
    "ShoppingListEntryBulkCreate",
    "ShoppingListEntrySimpleCreate",
    "ShoppingListRecipe",
    "SourceImportDuplicate",
    "SourceImportFood",
    "SourceImportIngredient",
    "SourceImportKeyword",
    "SourceImportProperty",
    "SourceImportPropertyType",
    "SourceImportRecipe",
    "SourceImportStep",
    "SourceImportUnit",
    "Space",
    "SpaceNavTextColorEnum",
    "SpaceThemeEnum",
    "Step",
    "Storage",
    "Supermarket",
    "SupermarketCategory",
    "SupermarketCategoryRelation",
    "Sync",
    "SyncLog",
    "ThemeEnum",
    "Unit",
    "UnitConversion",
    "User",
    "UserFile",
    "UserFileView",
    "UserPreference",
    "UserPreferenceNavTextColorEnum",
    "UserSpace",
    "ViewLog",
)
